@@ -1,4 +1,4 @@
-# Copyright 2020 QuantumBlack Visual Analytics Limited
+# Copyright 2021 QuantumBlack Visual Analytics Limited
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -39,17 +39,17 @@ from typing import Sequence
 import click
 from click import secho
 
-from kedro.framework.cli.jupyter import _load_project_context
 from kedro.framework.cli.utils import (
     KedroCliError,
     _check_module_importable,
     call,
+    command_with_verbosity,
     env_option,
     forward_command,
     ipython_message,
     python_call,
 )
-from kedro.framework.context import get_static_project_data
+from kedro.framework.startup import ProjectMetadata
 
 NO_DEPENDENCY_MESSAGE = """{module} is not installed. Please make sure {module} is in
 {src}/requirements.txt and run `kedro install`."""
@@ -76,22 +76,19 @@ def _build_reqs(source_path: Path, args: Sequence[str] = ()):
     python_call("piptools", ["compile", "-q", *args, str(requirements_in)])
 
 
-def _get_source_path():
-    return get_static_project_data(Path.cwd())["source_dir"]
-
-
 @click.group()
 def project_group():
     """Collection of project commands."""
 
 
 @forward_command(project_group, forward_help=True)
-def test(args):
+@click.pass_obj  # this will pass the metadata as first argument
+def test(metadata: ProjectMetadata, args, **kwargs):  # pylint: disable=unused-argument
     """Run the test suite."""
     try:
         _check_module_importable("pytest")
     except KedroCliError as exc:
-        source_path = _get_source_path()
+        source_path = metadata.source_dir
         raise KedroCliError(
             NO_DEPENDENCY_MESSAGE.format(module="pytest", src=str(source_path))
         ) from exc
@@ -99,16 +96,16 @@ def test(args):
         python_call("pytest", args)
 
 
-@project_group.command()
+@command_with_verbosity(project_group)
 @click.option("-c", "--check-only", is_flag=True, help=LINT_CHECK_ONLY_HELP)
 @click.argument("files", type=click.Path(exists=True), nargs=-1)
-def lint(files, check_only):
+@click.pass_obj  # this will pass the metadata as first argument
+def lint(
+    metadata: ProjectMetadata, files, check_only, **kwargs
+):  # pylint: disable=unused-argument
     """Run flake8, isort and black."""
-    static_data = get_static_project_data(Path.cwd())
-    source_path = static_data["source_dir"]
-    package_name = (
-        static_data.get("package_name") or _load_project_context().package_name
-    )
+    source_path = metadata.source_dir
+    package_name = metadata.package_name
     files = files or (str(source_path / "tests"), str(source_path / package_name))
 
     if "PYTHONPATH" not in os.environ:
@@ -125,12 +122,10 @@ def lint(files, check_only):
             ) from exc
 
     python_call("black", ("--check",) + files if check_only else files)
-    python_call("flake8", ("--max-line-length=88",) + files)
+    python_call("flake8", files)
 
     check_flag = ("-c",) if check_only else ()
-    python_call(
-        "isort", (*check_flag, "-rc", "-tc", "-up", "-fgw=0", "-m=3", "-w=88") + files
-    )
+    python_call("isort", (*check_flag, "-rc") + files)  # type: ignore
 
 
 @project_group.command()
@@ -141,13 +136,14 @@ def lint(files, check_only):
     help="Run `pip-compile` on project requirements before install. "
     "By default runs only if `src/requirements.in` file doesn't exist.",
 )
-def install(compile_flag):
+@click.pass_obj  # this will pass the metadata as first argument
+def install(metadata: ProjectMetadata, compile_flag):
     """Install project dependencies from both requirements.txt
     and environment.yml (optional)."""
     # we cannot use `context.project_path` as in other commands since
     # context instantiation might break due to missing dependencies
     # we attempt to install here
-    source_path = _get_source_path()
+    source_path = metadata.source_dir
     environment_yml = source_path / "environment.yml"
     requirements_in = source_path / "requirements.in"
     requirements_txt = source_path / "requirements.txt"
@@ -166,18 +162,26 @@ def install(compile_flag):
         python_call("pip", pip_command)
     else:
         command = [sys.executable, "-m", "pip"] + pip_command
-        subprocess.Popen(command, creationflags=subprocess.CREATE_NEW_CONSOLE)
+        proc = subprocess.Popen(
+            command, creationflags=subprocess.CREATE_NEW_CONSOLE, stderr=subprocess.PIPE
+        )
+        _, errs = proc.communicate()
+        if errs:
+            secho(errs.decode(), fg="red")
+            raise click.exceptions.Exit(code=1)
     secho("Requirements installed!", fg="green")
 
 
 @forward_command(project_group, forward_help=True)
 @env_option
-def ipython(env, args):
+@click.pass_obj  # this will pass the metadata as first argument
+def ipython(
+    metadata: ProjectMetadata, env, args, **kwargs
+):  # pylint: disable=unused-argument
     """Open IPython with project specific variables loaded."""
-    context = _load_project_context(env=env)
     _check_module_importable("IPython")
 
-    os.environ["IPYTHONDIR"] = str(context.project_path / ".ipython")
+    os.environ["IPYTHONDIR"] = str(metadata.project_path / ".ipython")
     if env:
         os.environ["KEDRO_ENV"] = env
     if "-h" not in args and "--help" not in args:
@@ -186,9 +190,10 @@ def ipython(env, args):
 
 
 @project_group.command()
-def package():
+@click.pass_obj  # this will pass the metadata as first argument
+def package(metadata: ProjectMetadata):
     """Package the project as a Python egg and wheel."""
-    source_path = _get_source_path()
+    source_path = metadata.source_dir
     call(
         [sys.executable, "setup.py", "clean", "--all", "bdist_egg"],
         cwd=str(source_path),
@@ -209,13 +214,11 @@ def package():
     default=False,
     help=OPEN_ARG_HELP,
 )
-def build_docs(open_docs):
+@click.pass_obj  # this will pass the metadata as first argument
+def build_docs(metadata: ProjectMetadata, open_docs):
     """Build the project documentation."""
-    static_data = get_static_project_data(Path.cwd())
-    source_path = static_data["source_dir"]
-    package_name = (
-        static_data.get("package_name") or _load_project_context().package_name
-    )
+    source_path = metadata.source_dir
+    package_name = metadata.package_name
 
     python_call("pip", ["install", str(source_path / "[docs]")])
     python_call("pip", ["install", "-r", str(source_path / "requirements.txt")])
@@ -238,9 +241,12 @@ def build_docs(open_docs):
 
 
 @forward_command(project_group, name="build-reqs")
-def build_reqs(args):
+@click.pass_obj  # this will pass the metadata as first argument
+def build_reqs(
+    metadata: ProjectMetadata, args, **kwargs
+):  # pylint: disable=unused-argument
     """Build the project dependency requirements."""
-    source_path = _get_source_path()
+    source_path = metadata.source_dir
     _build_reqs(source_path, args)
     secho(
         "Requirements built! Please update requirements.in "
@@ -250,10 +256,13 @@ def build_reqs(args):
     )
 
 
-@project_group.command("activate-nbstripout")
-def activate_nbstripout():
+@command_with_verbosity(project_group, "activate-nbstripout")
+@click.pass_obj  # this will pass the metadata as first argument
+def activate_nbstripout(
+    metadata: ProjectMetadata, **kwargs
+):  # pylint: disable=unused-argument
     """Install the nbstripout git hook to automatically clean notebooks."""
-    source_path = _get_source_path()
+    source_path = metadata.source_dir
     secho(
         (
             "Notebook output cells will be automatically cleared before committing"
