@@ -1,4 +1,4 @@
-# Copyright 2020 QuantumBlack Visual Analytics Limited
+# Copyright 2021 QuantumBlack Visual Analytics Limited
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -27,44 +27,45 @@
 # limitations under the License.
 """This module provides context for Kedro project."""
 
-import abc
+import functools
 import logging
-import logging.config
 import os
-import sys
 from copy import deepcopy
 from pathlib import Path, PurePosixPath, PureWindowsPath
-from typing import Any, Dict, Iterable, Tuple, Union
+from typing import Any, Dict, Iterable, Union
 from urllib.parse import urlparse
 from warnings import warn
 
-import anyconfig
-
-from kedro import __version__
 from kedro.config import ConfigLoader, MissingConfigException
 from kedro.framework.hooks import get_hook_manager
+from kedro.framework.project import settings
+from kedro.framework.startup import _get_project_metadata
 from kedro.io import DataCatalog
 from kedro.io.core import generate_timestamp
 from kedro.pipeline import Pipeline
 from kedro.pipeline.pipeline import _transcode_split
 from kedro.runner.runner import AbstractRunner
 from kedro.runner.sequential_runner import SequentialRunner
-from kedro.utils import load_obj
 from kedro.versioning import Journal
 
-_PLUGIN_HOOKS = "kedro.hooks"  # entry-point to load hooks from for installed plugins
 
-# Kedro configuration files in the precedence order
-KEDRO_CONFIGS = (".kedro.yml", "pyproject.toml")
+def _deprecate(version):
+    """Decorator to deprecate a few of the context's properties
+    """
 
+    def decorator(func):
+        @functools.wraps(func)
+        def wrapper(*args, **kwargs):
+            property_name = func.__name__
+            warn(
+                f"Accessing {property_name} via the context will be deprecated in Kedro {version}.",
+                DeprecationWarning,
+            )
+            return func(*args, **kwargs)
 
-def _version_mismatch_error(context_version) -> str:
-    return (
-        "Your Kedro project version {} does not match Kedro package version {} "
-        "you are running. Make sure to update your project template. See "
-        "https://github.com/quantumblacklabs/kedro/blob/master/RELEASE.md "
-        "for how to migrate your Kedro project."
-    ).format(context_version, __version__)
+        return wrapper
+
+    return decorator
 
 
 def _is_relative_path(path_string: str) -> bool:
@@ -136,7 +137,7 @@ def _convert_paths_to_absolute_posix(
     """
     if not project_path.is_absolute():
         raise ValueError(
-            "project_path must be an absolute path. Received: {}".format(project_path)
+            f"project_path must be an absolute path. Received: {project_path}"
         )
 
     # only check a few conf keys that are known to specify a path string as value
@@ -194,36 +195,18 @@ def _validate_layers_for_transcoding(catalog: DataCatalog) -> None:
         )
 
 
-class KedroContext(abc.ABC):
+class KedroContext:
     """``KedroContext`` is the base class which holds the configuration and
-    Kedro's main functionality. Project-specific context class should extend
-    this abstract class and implement the all abstract methods.
-
-    Attributes:
-       CONF_ROOT: Name of root directory containing project configuration.
-       Default name is "conf".
-       hooks: The list of hooks provided by user to extend KedroContext's execution.
-
-    Example:
-    ::
-
-        >>> from kedro.framework.context import KedroContext
-        >>> from kedro.pipeline import Pipeline
-        >>>
-        >>> class ProjectContext(KedroContext):
-        >>>     @property
-        >>>     def pipeline(self) -> Pipeline:
-        >>>         return Pipeline([])
-
+    Kedro's main functionality.
     """
 
-    CONF_ROOT = "conf"
-
-    # Registry for user-defined hooks to be overwritten by a project context.
-    hooks: Tuple = ()
+    _CONF_ROOT = "conf"
+    """CONF_ROOT: Name of root directory containing project configuration.
+    Default name is "conf"."""
 
     def __init__(
         self,
+        package_name: str,
         project_path: Union[Path, str],
         env: str = None,
         extra_params: Dict[str, Any] = None,
@@ -236,6 +219,8 @@ class KedroContext(abc.ABC):
                 between Kedro project version and package version.
 
         Args:
+            package_name: Package name for the Kedro project the context is
+                created for.
             project_path: Project path to define the context for.
             env: Optional argument for configuration default environment to be used
                 for running the pipeline. If not specified, it defaults to "local".
@@ -244,50 +229,44 @@ class KedroContext(abc.ABC):
                 the parameters retrieved from the project configuration.
         """
         self._project_path = Path(project_path).expanduser().resolve()
-        self._static_data = get_static_project_data(self._project_path)
+        self._package_name = package_name
 
-        # check the match for major and minor version (skip patch version)
-        if self.project_version.split(".")[:2] != __version__.split(".")[:2]:
-            raise KedroContextError(_version_mismatch_error(self.project_version))
-
-        self.env = env or "local"
+        self._env = env or "local"
         self._extra_params = deepcopy(extra_params)
 
-        self._register_hooks(auto=True)
-        # we need a ConfigLoader registered in order to be able to set up logging
-        self._setup_logging()
-
-    @property
-    def static_data(self) -> Dict[str, Any]:
-        """Read-only property for Kedro project static data.
+    @property  # type: ignore
+    @_deprecate(version="0.18.0")
+    def CONF_ROOT(self) -> str:  # pylint: disable=invalid-name
+        """Deprecated in favour of settings.CONF_ROOT
 
         Returns:
-            A dictionary with defined static project data.
-
+            The root directory of the configuration directory of the project.
+        Raises:
+            DeprecationWarning
         """
-        return deepcopy(self._static_data)
+        return self._CONF_ROOT
 
-    @property
-    def project_name(self) -> str:
-        """Property for Kedro project name.
+    @CONF_ROOT.setter  # type: ignore
+    @_deprecate(version="0.18.0")
+    def CONF_ROOT(self, value: str) -> None:  # pylint: disable=invalid-name
+        """Deprecated in favour of settings.CONF_ROOT
+        Raises:
+            DeprecationWarning
+        """
+        self._CONF_ROOT = value  # pylint: disable=invalid-name
+
+    @property  # type: ignore
+    def env(self) -> str:
+        """Property for the current Kedro environment.
 
         Returns:
-            Name of Kedro project.
+            Name of the current Kedro environment.
 
         """
-        return self.static_data["project_name"]
+        return self._env
 
-    @property
-    def project_version(self) -> str:
-        """Property for Kedro version.
-
-        Returns:
-            Kedro version.
-
-        """
-        return self.static_data["project_version"]
-
-    @property
+    @property  # type: ignore
+    @_deprecate(version="0.18.0")
     def package_name(self) -> str:
         """Property for Kedro project package name.
 
@@ -295,7 +274,7 @@ class KedroContext(abc.ABC):
             Name of Kedro project package.
 
         """
-        return self.static_data["package_name"]
+        return self._package_name
 
     @property
     def pipeline(self) -> Pipeline:
@@ -316,66 +295,6 @@ class KedroContext(abc.ABC):
         """
         return self._get_pipelines()
 
-    def _register_hooks_setuptools(self):
-        """Register pluggy hooks from setuptools entrypoints."""
-        hook_manager = get_hook_manager()
-        already_registered = hook_manager.get_plugins()
-        found = hook_manager.load_setuptools_entrypoints(_PLUGIN_HOOKS)
-        disable_plugins = set(self.static_data.get("disable_hooks_for_plugins", []))
-
-        # Get list of plugin/distinfo tuples for all setuptools registered plugins.
-        plugininfo = hook_manager.list_plugin_distinfo()
-        plugin_names = []
-        disabled_plugin_names = []
-        for plugin, dist in plugininfo:
-            if dist.project_name in disable_plugins:
-                # `unregister()` is used instead of `set_blocked()` because
-                # we want to disable hooks for specific plugin based on project
-                # name and not `entry_point` name. Also, we log project names with
-                # version for which hooks were registered.
-                hook_manager.unregister(plugin=plugin)
-                found -= 1
-                disabled_plugin_names.append(f"{dist.project_name}-{dist.version}")
-            elif plugin not in already_registered:
-                plugin_names.append(f"{dist.project_name}-{dist.version}")
-
-        if disabled_plugin_names:
-            logging.info(
-                "Hooks are disabled for plugin(s): %s",
-                ", ".join(sorted(disabled_plugin_names)),
-            )
-
-        if plugin_names:
-            logging.info(
-                "Registered hooks from %d installed plugin(s): %s",
-                found,
-                ", ".join(sorted(plugin_names)),
-            )
-
-    def _register_hooks(self, auto: bool = False) -> None:
-        """Register all hooks as specified in ``hooks`` with the global ``hook_manager``,
-        and, optionally, from installed plugins.
-
-        Args:
-            auto: An optional flag to enable auto-discovery and registration of plugin hooks.
-        """
-        hook_manager = get_hook_manager()
-
-        # enrich with hooks specified in .kedro.yml or pyproject.toml if .kedro.yml doesn't exist
-        hooks_locations = self.static_data.get("hooks", [])
-        configured_hooks = tuple(load_obj(hook) for hook in hooks_locations)
-
-        all_hooks = self.hooks + configured_hooks
-        for hooks_collection in all_hooks:
-            # Sometimes users might create more than one context instance, in which case
-            # hooks have already been registered, so we perform a simple check here
-            # to avoid an error being raised and break user's workflow.
-            if not hook_manager.is_registered(hooks_collection):
-                hook_manager.register(hooks_collection)
-
-        if auto:
-            self._register_hooks_setuptools()
-
     def _get_pipeline(self, name: str = None) -> Pipeline:
         name = name or "__default__"
         pipelines = self._get_pipelines()
@@ -428,6 +347,8 @@ class KedroContext(abc.ABC):
 
         Returns:
             DataCatalog defined in `catalog.yml`.
+        Raises:
+            KedroContextError: Incorrect ``DataCatalog`` registered for the project.
 
         """
         return self._get_catalog()
@@ -446,11 +367,7 @@ class KedroContext(abc.ABC):
                 "parameters*", "parameters*/**", "**/parameters*"
             )
         except MissingConfigException as exc:
-            warn(
-                "Parameters not found in your Kedro project config.\n{}".format(
-                    str(exc)
-                )
-            )
+            warn(f"Parameters not found in your Kedro project config.\n{str(exc)}")
             params = {}
         params.update(self._extra_params or {})
         return params
@@ -465,6 +382,8 @@ class KedroContext(abc.ABC):
 
         Returns:
             DataCatalog defined in `catalog.yml`.
+        Raises:
+            KedroContextError: Incorrect ``DataCatalog`` registered for the project.
 
         """
         # '**/catalog*' reads modular pipeline configs
@@ -475,9 +394,21 @@ class KedroContext(abc.ABC):
             project_path=self.project_path, conf_dictionary=conf_catalog
         )
         conf_creds = self._get_config_credentials()
-        catalog = self._create_catalog(
-            conf_catalog, conf_creds, save_version, journal, load_versions
+
+        hook_manager = get_hook_manager()
+        catalog = hook_manager.hook.register_catalog(  # pylint: disable=no-member
+            catalog=conf_catalog,
+            credentials=conf_creds,
+            load_versions=load_versions,
+            save_version=save_version,
+            journal=journal,
         )
+        if not isinstance(catalog, DataCatalog):
+            raise KedroContextError(
+                f"Expected an instance of `DataCatalog`, "
+                f"got `{type(catalog).__name__}` instead."
+            )
+
         feed_dict = self._get_feed_dict()
         catalog.add_feed_dict(feed_dict)
         if catalog.layers:
@@ -494,33 +425,6 @@ class KedroContext(abc.ABC):
         )
         return catalog
 
-    def _create_catalog(  # pylint: disable=no-self-use,too-many-arguments
-        self,
-        conf_catalog: Dict[str, Any],
-        conf_creds: Dict[str, Any],
-        save_version: str = None,
-        journal: Journal = None,
-        load_versions: Dict[str, str] = None,
-    ) -> DataCatalog:
-        """A factory method for the DataCatalog instantiation.
-
-        Returns:
-            DataCatalog defined in `catalog.yml`.
-
-        """
-        hook_manager = get_hook_manager()
-        catalog = hook_manager.hook.register_catalog(  # pylint: disable=no-member
-            catalog=conf_catalog,
-            credentials=conf_creds,
-            load_versions=load_versions,
-            save_version=save_version,
-            journal=journal,
-        )
-
-        return catalog or DataCatalog.from_config(  # for backwards compatibility
-            conf_catalog, conf_creds, load_versions, save_version, journal
-        )
-
     @property
     def io(self) -> DataCatalog:
         """Read-only alias property referring to Kedro's ``DataCatalog`` for this
@@ -528,38 +432,37 @@ class KedroContext(abc.ABC):
 
         Returns:
             DataCatalog defined in `catalog.yml`.
+        Raises:
+            KedroContextError: Incorrect ``DataCatalog`` registered for the project.
 
         """
         # pylint: disable=invalid-name
         return self.catalog
 
-    def _create_config_loader(  # pylint: disable=no-self-use
-        self, conf_paths: Iterable[str]
-    ) -> ConfigLoader:
-        """A factory method for the ConfigLoader instantiation.
-
-        Returns:
-            Instance of `ConfigLoader`.
-
-        """
-        hook_manager = get_hook_manager()
-        config_loader = hook_manager.hook.register_config_loader(  # pylint: disable=no-member
-            conf_paths=conf_paths
-        )
-        return config_loader or ConfigLoader(conf_paths)  # for backwards compatibility
-
     def _get_config_loader(self) -> ConfigLoader:
         """A hook for changing the creation of a ConfigLoader instance.
 
         Returns:
-            Instance of `ConfigLoader` created by `_create_config_loader()`.
+            Instance of `ConfigLoader` created by `register_config_loader` hook.
+        Raises:
+            KedroContextError: Incorrect ``ConfigLoader`` registered for the project.
 
         """
+        conf_root = settings.CONF_ROOT
         conf_paths = [
-            str(self.project_path / self.CONF_ROOT / "base"),
-            str(self.project_path / self.CONF_ROOT / self.env),
+            str(self.project_path / conf_root / "base"),
+            str(self.project_path / conf_root / self.env),
         ]
-        return self._create_config_loader(conf_paths)
+        hook_manager = get_hook_manager()
+        config_loader = hook_manager.hook.register_config_loader(  # pylint: disable=no-member
+            conf_paths=conf_paths, env=self.env, extra_params=self._extra_params,
+        )
+        if not isinstance(config_loader, ConfigLoader):
+            raise KedroContextError(
+                f"Expected an instance of `ConfigLoader`, "
+                f"got `{type(config_loader).__name__}` instead."
+            )
+        return config_loader
 
     @property
     def config_loader(self) -> ConfigLoader:
@@ -568,20 +471,12 @@ class KedroContext(abc.ABC):
 
         Returns:
             Instance of `ConfigLoader`.
+        Raises:
+            KedroContextError: Incorrect ``ConfigLoader`` registered for the project.
 
         """
         return self._get_config_loader()
 
-    def _setup_logging(self) -> None:
-        """Register logging specified in logging directory."""
-        conf_logging = self.config_loader.get("logging*", "logging*/**", "**/logging*")
-        # turn relative paths in logging config into absolute path
-        # before initialising loggers
-        conf_logging = _convert_paths_to_absolute_posix(
-            project_path=self.project_path, conf_dictionary=conf_logging
-        )
-        logging.config.dictConfig(conf_logging)
-
     def _get_feed_dict(self) -> Dict[str, Any]:
         """Get parameters and return the feed dictionary."""
         params = self.params
@@ -600,12 +495,12 @@ class KedroContext(abc.ABC):
                 >>> assert feed_dict["params:a"] == {"b": 1}
                 >>> assert feed_dict["params:a.b"] == 1
             """
-            key = "params:{}".format(param_name)
+            key = f"params:{param_name}"
             feed_dict[key] = param_value
 
             if isinstance(param_value, dict):
                 for key, val in param_value.items():
-                    _add_param_to_feed_dict("{}.{}".format(param_name, key), val)
+                    _add_param_to_feed_dict(f"{param_name}.{key}", val)
 
         for param_name, param_value in params.items():
             _add_param_to_feed_dict(param_name, param_value)
@@ -619,11 +514,7 @@ class KedroContext(abc.ABC):
                 "credentials*", "credentials*/**", "**/credentials*"
             )
         except MissingConfigException as exc:
-            warn(
-                "Credentials not found in your Kedro project config.\n{}".format(
-                    str(exc)
-                )
-            )
+            warn(f"Credentials not found in your Kedro project config.\n{str(exc)}")
             conf_creds = {}
         return conf_creds
 
@@ -636,6 +527,7 @@ class KedroContext(abc.ABC):
         to_nodes: Iterable[str] = None,
         node_names: Iterable[str] = None,
         from_inputs: Iterable[str] = None,
+        to_outputs: Iterable[str] = None,
     ) -> Pipeline:
         """Filter the pipeline as the intersection of all conditions."""
         new_pipeline = pipeline
@@ -648,7 +540,7 @@ class KedroContext(abc.ABC):
             new_pipeline &= pipeline.only_nodes_with_tags(*tags)
             if not new_pipeline.nodes:
                 raise KedroContextError(
-                    "Pipeline contains no nodes with tags: {}".format(str(tags))
+                    f"Pipeline contains no nodes with tags: {str(tags)}"
                 )
         if from_nodes:
             new_pipeline &= pipeline.from_nodes(*from_nodes)
@@ -658,6 +550,8 @@ class KedroContext(abc.ABC):
             new_pipeline &= pipeline.only_nodes(*node_names)
         if from_inputs:
             new_pipeline &= pipeline.from_inputs(*from_inputs)
+        if to_outputs:
+            new_pipeline &= pipeline.to_outputs(*to_outputs)
 
         if not new_pipeline.nodes:
             raise KedroContextError("Pipeline contains no nodes")
@@ -678,6 +572,7 @@ class KedroContext(abc.ABC):
         from_nodes: Iterable[str] = None,
         to_nodes: Iterable[str] = None,
         from_inputs: Iterable[str] = None,
+        to_outputs: Iterable[str] = None,
         load_versions: Dict[str, str] = None,
         pipeline_name: str = None,
     ) -> Dict[str, Any]:
@@ -698,6 +593,8 @@ class KedroContext(abc.ABC):
                 end point of the new ``Pipeline``.
             from_inputs: An optional list of input datasets which should be used as a
                 starting point of the new ``Pipeline``.
+            to_outputs: An optional list of output datasets which should be used as an
+                end point of the new ``Pipeline``.
             load_versions: An optional flag to specify a particular dataset version timestamp
                 to load.
             pipeline_name: Name of the ``Pipeline`` to execute.
@@ -723,6 +620,7 @@ class KedroContext(abc.ABC):
             to_nodes=to_nodes,
             node_names=node_names,
             from_inputs=from_inputs,
+            to_outputs=to_outputs,
         )
 
         save_version = self._get_save_version()
@@ -732,12 +630,12 @@ class KedroContext(abc.ABC):
             "run_id": run_id,
             "project_path": str(self.project_path),
             "env": self.env,
-            "kedro_version": self.project_version,
             "tags": tags,
             "from_nodes": from_nodes,
             "to_nodes": to_nodes,
             "node_names": node_names,
             "from_inputs": from_inputs,
+            "to_outputs": to_outputs,
             "load_versions": load_versions,
             "pipeline_name": pipeline_name,
             "extra_params": self._extra_params,
@@ -793,128 +691,6 @@ class KedroContext(abc.ABC):
         return generate_timestamp()
 
 
-def get_static_project_data(project_path: Union[str, Path]) -> Dict[str, Any]:
-    """Read static project data from `<project_path>/.kedro.yml` config file if it
-    exists otherwise from `<project_path>/pyproject.toml` (under `[tool.kedro]` section).
-
-    Args:
-        project_path: Local path to project root directory to look up `.kedro.yml` or
-            `pyproject.toml` in.
-
-    Raises:
-        KedroContextError: Neither '.kedro.yml' nor `pyproject.toml` was found
-            or `[tool.kedro]` section is missing in `pyproject.toml`, or config file
-            cannot be parsed.
-
-    Returns:
-        A mapping that contains static project data.
-    """
-    project_path = Path(project_path).expanduser().resolve()
-
-    config_paths = [
-        project_path / conf_file
-        for conf_file in KEDRO_CONFIGS
-        if (project_path / conf_file).is_file()
-    ]
-
-    if not config_paths:
-        configs = ", ".join(KEDRO_CONFIGS)
-        raise KedroContextError(
-            f"Could not find any of configuration files '{configs}' in {project_path}. "
-            f"If you have created your project with Kedro "
-            f"version <0.15.0, make sure to update your project template. "
-            f"See https://github.com/quantumblacklabs/kedro/blob/master/RELEASE.md"
-            f"#migration-guide-from-kedro-014-to-kedro-0150 "
-            f"for how to migrate your Kedro project."
-        )
-
-    # First found wins
-    kedro_config = config_paths[0]
-    try:
-        static_data = anyconfig.load(kedro_config)
-    except Exception as exc:
-        raise KedroContextError(f"Failed to parse '{kedro_config.name}' file.") from exc
-
-    if kedro_config.suffix == ".toml":
-        try:
-            static_data = static_data["tool"]["kedro"]
-        except KeyError as exc:
-            raise KedroContextError(
-                f"There's no '[tool.kedro]' section in the '{kedro_config.name}'. "
-                f"Please add '[tool.kedro]' section to the file with appropriate "
-                f"configuration parameters."
-            ) from exc
-
-    source_dir = Path(static_data.get("source_dir", "src")).expanduser()
-    source_dir = (project_path / source_dir).resolve()
-    static_data["source_dir"] = source_dir
-    static_data["config_file"] = kedro_config
-
-    return static_data
-
-
-def validate_source_path(source_path: Path, project_path: Path):
-    """Validate the source path exists and is relative to the project path.
-
-    Args:
-        source_path: Absolute source path.
-        project_path: Path to the Kedro project.
-
-    Raises:
-        KedroContextError: Either source_path is not relative to project_path or
-            source_path does not exist.
-
-    """
-    try:
-        source_path.relative_to(project_path)
-    except ValueError as exc:
-        raise KedroContextError(
-            f"Source path '{source_path}' has to be relative to "
-            f"your project root '{project_path}'."
-        ) from exc
-    if not source_path.exists():
-        raise KedroContextError(f"Source path '{source_path}' cannot be found.")
-
-
-def load_package_context(
-    project_path: Path, package_name: str, **kwargs
-) -> KedroContext:
-    """Loads the KedroContext object of a Kedro project package,
-    as output by `kedro package` and installed via `pip`.
-    This function is only intended to be used in a project's `run.py`.
-    If you are looking to load KedroContext object for any other workflow,
-    you might want to use ``load_context`` instead.
-
-    Args:
-        project_path: Path to the Kedro project, i.e. where `conf/` resides.
-        package_name: Name of the installed Kedro project package.
-        kwargs: Optional kwargs for ``ProjectContext`` class in `run.py`.
-
-    Returns:
-        Instance of ``KedroContext`` class defined in Kedro project.
-
-    Raises:
-        KedroContextError: Neither '.kedro.yml' nor `pyproject.toml` was found
-            or `[tool.kedro]` section is missing in `pyproject.toml`, or loaded context
-            has package conflict.
-    """
-    context_path = f"{package_name}.run.ProjectContext"
-    try:
-        context_class = load_obj(context_path)
-    except ModuleNotFoundError as exc:
-        raise KedroContextError(
-            f"Cannot load context object from {context_path} for package {package_name}."
-        ) from exc
-
-    # update kwargs with env from the environment variable (defaults to None if not set)
-    # need to do this because some CLI command (e.g `kedro run`) defaults to passing
-    # in `env=None`
-    kwargs["env"] = kwargs.get("env") or os.getenv("KEDRO_ENV")
-
-    context = context_class(project_path=project_path, **kwargs)
-    return context
-
-
 def load_context(project_path: Union[str, Path], **kwargs) -> KedroContext:
     """Loads the KedroContext object of a Kedro Project.
     This is the default way to load the KedroContext object for normal workflows such as
@@ -923,56 +699,40 @@ def load_context(project_path: Union[str, Path], **kwargs) -> KedroContext:
 
        <project_path>
            |__ <src_dir>
-           |__ .kedro.yml
-           |__ kedro_cli.py
            |__ pyproject.toml
 
-    The name of the <scr_dir> is `src` by default. The `.kedro.yml` or `pyproject.toml` can
-    be used for configuration. If `.kedro.yml` exists, it will be used otherwise, `pyproject.toml`
-    will be treated as the configuration file (Kedro configuration should be under
-    `[tool.kedro]` section).
+    The name of the <scr_dir> is `src` by default. The `pyproject.toml` file is used
+    for project metadata. Kedro configuration should be under `[tool.kedro]` section.
 
     Args:
         project_path: Path to the Kedro project.
-        kwargs: Optional kwargs for ``ProjectContext`` class in `run.py`.
+        kwargs: Optional kwargs for ``KedroContext`` class.
 
     Returns:
         Instance of ``KedroContext`` class defined in Kedro project.
 
     Raises:
-        KedroContextError: Neither '.kedro.yml' nor `pyproject.toml` was found
-            or `[tool.kedro]` section is missing in `pyproject.toml`, or loaded context
-            has package conflict.
+        KedroContextError: `pyproject.toml` was not found or the `[tool.kedro]` section
+            is missing, or loaded context has package conflict.
 
     """
+    warn(
+        "`kedro.framework.context.load_context` is now deprecated in favour of "
+        "`KedroSession.load_context` and will be removed in Kedro 0.18.0.",
+        DeprecationWarning,
+    )
     project_path = Path(project_path).expanduser().resolve()
-    static_data = get_static_project_data(project_path)
-
-    source_dir = static_data["source_dir"]
-    validate_source_path(source_dir, project_path)
-
-    if "context_path" not in static_data:
-        conf_file = static_data["config_file"].name
-        raise KedroContextError(
-            f"'{conf_file}' doesn't have a required `context_path` field. "
-            f"Please refer to the documentation."
-        )
-
-    if str(source_dir) not in sys.path:
-        sys.path.insert(0, str(source_dir))
-
-    if "PYTHONPATH" not in os.environ:
-        os.environ["PYTHONPATH"] = str(source_dir)
-
-    context_class = load_obj(static_data["context_path"])
+    metadata = _get_project_metadata(project_path)
 
+    context_class = settings.CONTEXT_CLASS
     # update kwargs with env from the environment variable
     # (defaults to None if not set)
     # need to do this because some CLI command (e.g `kedro run`) defaults to
     # passing in `env=None`
     kwargs["env"] = kwargs.get("env") or os.getenv("KEDRO_ENV")
-
-    context = context_class(project_path=project_path, **kwargs)
+    context = context_class(
+        package_name=metadata.package_name, project_path=project_path, **kwargs
+    )
     return context
 
 
