@@ -1,4 +1,4 @@
-# Copyright 2020 QuantumBlack Visual Analytics Limited
+# Copyright 2021 QuantumBlack Visual Analytics Limited
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -85,8 +85,7 @@ class AbstractRunner(ABC):
         unsatisfied = pipeline.inputs() - set(catalog.list())
         if unsatisfied:
             raise ValueError(
-                "Pipeline input(s) {} not found in the "
-                "DataCatalog".format(unsatisfied)
+                f"Pipeline input(s) {unsatisfied} not found in the DataCatalog"
             )
 
         free_outputs = pipeline.outputs() - set(catalog.list())
@@ -180,7 +179,7 @@ class AbstractRunner(ABC):
 
             start_p = resume_p.only_nodes_with_inputs(*resume_p.inputs())
             start_node_names = (n.name for n in start_p.nodes)
-            postfix += '  --from-nodes "{}"'.format(",".join(start_node_names))
+            postfix += f"  --from-nodes \"{','.join(start_node_names)}\""
 
         self._logger.warning(
             "There are %d nodes that have not run.\n"
@@ -217,13 +216,41 @@ def run_node(
     return node
 
 
-def _run_node_sequential(node: Node, catalog: DataCatalog, run_id: str = None) -> Node:
-    inputs = {name: catalog.load(name) for name in node.inputs}
+def _collect_inputs_from_hook(
+    node: Node,
+    catalog: DataCatalog,
+    inputs: Dict[str, Any],
+    is_async: bool,
+    run_id: str = None,
+) -> Dict[str, Any]:
+    inputs = inputs.copy()  # shallow copy to prevent in-place modification by the hook
     hook_manager = get_hook_manager()
-    is_async = False
-    hook_manager.hook.before_node_run(  # pylint: disable=no-member
-        node=node, catalog=catalog, inputs=inputs, is_async=is_async, run_id=run_id
+    hook_response = hook_manager.hook.before_node_run(  # pylint: disable=no-member
+        node=node, catalog=catalog, inputs=inputs, is_async=is_async, run_id=run_id,
     )
+
+    additional_inputs = {}
+    for response in hook_response:
+        if response is not None and not isinstance(response, dict):
+            response_type = type(response).__name__
+            raise TypeError(
+                f"`before_node_run` must return either None or a dictionary mapping "
+                f"dataset names to updated values, got `{response_type}` instead."
+            )
+        response = response or {}
+        additional_inputs.update(response)
+
+    return additional_inputs
+
+
+def _call_node_run(
+    node: Node,
+    catalog: DataCatalog,
+    inputs: Dict[str, Any],
+    is_async: bool,
+    run_id: str = None,
+) -> Dict[str, Any]:
+    hook_manager = get_hook_manager()
     try:
         outputs = node.run(inputs)
     except Exception as exc:
@@ -244,52 +271,79 @@ def _run_node_sequential(node: Node, catalog: DataCatalog, run_id: str = None) -
         is_async=is_async,
         run_id=run_id,
     )
+    return outputs
+
+
+def _run_node_sequential(node: Node, catalog: DataCatalog, run_id: str = None) -> Node:
+    inputs = {}
+    hook_manager = get_hook_manager()
+
+    for name in node.inputs:
+        hook_manager.hook.before_dataset_loaded(  # pylint: disable=no-member
+            dataset_name=name
+        )
+        inputs[name] = catalog.load(name)
+        hook_manager.hook.after_dataset_loaded(  # pylint: disable=no-member
+            dataset_name=name, data=inputs[name]
+        )
+
+    is_async = False
+
+    additional_inputs = _collect_inputs_from_hook(
+        node, catalog, inputs, is_async, run_id=run_id
+    )
+    inputs.update(additional_inputs)
+
+    outputs = _call_node_run(node, catalog, inputs, is_async, run_id=run_id)
 
     for name, data in outputs.items():
+        hook_manager.hook.before_dataset_saved(  # pylint: disable=no-member
+            dataset_name=name, data=data
+        )
         catalog.save(name, data)
+        hook_manager.hook.after_dataset_saved(  # pylint: disable=no-member
+            dataset_name=name, data=data
+        )
     return node
 
 
 def _run_node_async(node: Node, catalog: DataCatalog, run_id: str = None) -> Node:
     with ThreadPoolExecutor() as pool:
-        inputs = {
-            name: pool.submit(catalog.load, name) for name in node.inputs
-        }  # Python dict is thread-safe
+        inputs = {}
+        hook_manager = get_hook_manager()
+
+        for name in node.inputs:
+            hook_manager.hook.before_dataset_loaded(  # pylint: disable=no-member
+                dataset_name=name
+            )
+            inputs[name] = pool.submit(catalog.load, name)
+            hook_manager.hook.after_dataset_loaded(  # pylint: disable=no-member
+                dataset_name=name, data=inputs[name]
+            )
+
         wait(inputs.values(), return_when=ALL_COMPLETED)
         inputs = {key: value.result() for key, value in inputs.items()}
-        hook_manager = get_hook_manager()
         is_async = True
-        hook_manager.hook.before_node_run(  # pylint: disable=no-member
-            node=node, catalog=catalog, inputs=inputs, is_async=is_async, run_id=run_id
-        )
-        try:
-            outputs = node.run(inputs)
-        except Exception as exc:
-            hook_manager.hook.on_node_error(  # pylint: disable=no-member
-                error=exc,
-                node=node,
-                catalog=catalog,
-                inputs=inputs,
-                is_async=is_async,
-                run_id=run_id,
-            )
-            raise exc
-        hook_manager.hook.after_node_run(  # pylint: disable=no-member
-            node=node,
-            catalog=catalog,
-            inputs=inputs,
-            outputs=outputs,
-            is_async=is_async,
-            run_id=run_id,
+        additional_inputs = _collect_inputs_from_hook(
+            node, catalog, inputs, is_async, run_id=run_id
         )
+        inputs.update(additional_inputs)
+
+        outputs = _call_node_run(node, catalog, inputs, is_async, run_id=run_id)
 
         save_futures = set()
 
         for name, data in outputs.items():
+            hook_manager.hook.before_dataset_saved(  # pylint: disable=no-member
+                dataset_name=name, data=data
+            )
             save_futures.add(pool.submit(catalog.save, name, data))
 
         for future in as_completed(save_futures):
             exception = future.exception()
             if exception:
                 raise exception
+            hook_manager.hook.after_dataset_saved(  # pylint: disable=no-member
+                dataset_name=name, data=data  # pylint: disable=undefined-loop-variable
+            )
     return node
